;*********************************************************************
; 3031 CONTROLLER ~ MIDI to CV Converter                             *
;*********************************************************************
; (c) 2005 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Memory map, and constant declarations
;
;*********************************************************************


;==================================================
; MISC

#DEFINE			_C	STATUS,C
#DEFINE			_Z	STATUS,Z
LSB			EQU	0x0
MSB			EQU	0x7


;==================================================
; MUX

#DEFINE			MUX0	PORTA,1
#DEFINE			MUX1	PORTA,2
#DEFINE			MUX2	PORTA,3
#DEFINE			MUX3	PORTA,5
#DEFINE			MUXE	PORTC,4

;==================================================
; DAC

DACCS			EQU	0x2


;==================================================
; Gates

GATE1			EQU	0x0
GATE2			EQU	0x1
GATE3			EQU	0x2


;==================================================
; Memory - bank 0

; MIDI stuff
MIDI_FLAGS		EQU	0x0020
  WHICH_BYTE		EQU	0x0		; 0 - next data byte is 1st, 1 - next data byte is 2nd
  MSG_LEN		EQU	0x1		; message length: 0=2byte 1=3byte

MIDI_MSG_PROCESS_PTR	EQU	0x0021		; pointer to next MIDI message to process
MIDI_MSG_INCOMING_PTR	EQU	0x0022		; pointer to where to write next incoming MIDI message

MIDI_MSG1		EQU	0x0023		; temp buffer for message byte 1
MIDI_MSG2		EQU	0x0024		; temp buffer for message byte 2

NOTE_VELOCITY_STACK_PTR	EQU	0x0025		; Note + velocity stack pointer

EEPROM_DATA_WRITE	EQU	0x0026		; When this gets incremented to >= 64 by TMR1 then check if anything needs writing to EEPROM

MIDI_MSG_BUFFER		EQU	0x0040		; start of MIDI message ring buffer
;				0x0041		; (needs to be at this address for PointToNextIncomingBufferByte macro to work)
;				0x0042
;				0x0043
;				0x0044
;				0x0045
;				0x0046
;				0x0047
;				0x0048
;				0x0049
;				0x004A
;				0x004B
;				0x004C
;				0x004D
;				0x004E
MIDI_MSG_BUFFER_END	EQU	0x004F		; end of MIDI message ring buffer


; Max			EQU	0x005F


; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0  - used by interupts, so don't call maths routines in bank 0 from main loop!
; 0x00E0-0x00EF in bank 1
; 0x0160-0x016F in bank 2  - memory used for settings, don't call maths routines in bank 2!
; 0x01E0-0x01EF in bank 3
AARGB2			EQU	0x0060		; argument A, LSB
AARGB1			EQU	0x0061		; argument A
AARGB0			EQU	0x0062		; argument A, MSB
BARGB1			EQU	0x0063		; argument B, LSB
BARGB0			EQU	0x0064		; argument B, MSB
REMB1			EQU	0x0065		; remainder LSB
REMB0			EQU	0x0066		; remainder MSB
TEMP			EQU	0x0067
TEMPB3			EQU	0x0068
TEMPB1			EQU	0x0069
TEMPB0			EQU	0x006A
SIGN			EQU	0x006B
LOOPCOUNT		EQU	0x006C
FPFLAGS			EQU	0x006D
  NAN			EQU	0x0
AARGB5			EQU	0x006E		; Used only by UDIV1608 and UDIV1608L macros
SPARE0			EQU	0x006F


; Memory accessible in all banks

W_TEMP        		EQU     0x0070        	; variable used for context saving
STATUS_TEMP		EQU	0x0071		; variable used for context saving
PCLATH_TEMP		EQU	0x0072		; variable used for context saving
FSR_TEMP		EQU	0x0073		; variable used for context saving
COUNTER			EQU	0x0074		; counter for loops
TEMP1			EQU	0x0075		; general purpose temp variable
TEMP2			EQU	0x0076		; general purpose temp variable
TEMP3			EQU	0x0077		; general purpose temp variable
TEMP4			EQU	0x0078		; general purpose temp variable
TEMP5			EQU	0x0079		; general purpose temp variable

; Temp vars for main loop
MTEMP1			EQU	0x007A
MTEMP2			EQU	0x007B
MTEMP3			EQU	0x007C
MTEMP4			EQU	0x007D
MTEMP5			EQU	0x007E

; Max			EQU	0x007F


;==================================================
; Memory - bank 1

NEXT_CV			EQU	0x00A0		; 1-15

CV1_VALUE_L		EQU	0x00A2
CV1_VALUE_H		EQU	0x00A3
CV2_VALUE_L		EQU	0x00A4
CV2_VALUE_H		EQU	0x00A5
CV3_VALUE_L		EQU	0x00A6
CV3_VALUE_H		EQU	0x00A7
CV4_VALUE_L		EQU	0x00A8
CV4_VALUE_H		EQU	0x00A9
CV5_VALUE_L		EQU	0x00AA
CV5_VALUE_H		EQU	0x00AB
CV6_VALUE_L		EQU	0x00AC
CV6_VALUE_H		EQU	0x00AD
CV7_VALUE_L		EQU	0x00AE
CV7_VALUE_H		EQU	0x00AF
CV8_VALUE_L		EQU	0x00B0
CV8_VALUE_H		EQU	0x00B1
CV9_VALUE_L		EQU	0x00B2
CV9_VALUE_H		EQU	0x00B3
CV10_VALUE_L		EQU	0x00B4
CV10_VALUE_H		EQU	0x00B5
CV11_VALUE_L		EQU	0x00B6
CV11_VALUE_H		EQU	0x00B7
CV12_VALUE_L		EQU	0x00B8
CV12_VALUE_H		EQU	0x00B9
CV13_VALUE_L		EQU	0x00BA
CV13_VALUE_H		EQU	0x00BB
CV14_VALUE_L		EQU	0x00BC
CV14_VALUE_H		EQU	0x00BD
CV15_VALUE_L		EQU	0x00BE
CV15_VALUE_H		EQU	0x00BF

; Pitch bend pre-calculations
BEND_PRECALC1_L		EQU	0x00C0	; 0x2D25 + PBR * 0x03C1 / 3 >> 3
BEND_PRECALC1_H		EQU	0x00C1
BEND_PRECALC2_L		EQU	0x00C2	; PBR * 0x03C1 / 3
BEND_PRECALC2_H		EQU	0x00C3

; LFO variables
LFO_STEP_L		EQU	0x00C4	; Step for angle to increment per 768us cycle
LFO_STEP_H		EQU	0x00C5	; 3276 / (128 - LFO_Speed)
LFO_ANGLE_L		EQU	0x00C6
LFO_ANGLE_H		EQU	0x00C7
LFO_PRECALC		EQU	0x00C8	; 0x12C5 * LFO_Depth >> 13 (7 bit unsigned)


; Max			EQU	0x00DF

; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0
; 0x00E0-0x00EF in bank 1
; 0x01E0-0x01EF in bank 3


;==================================================
; Memory - bank 2

; MIDI set parameter values and their defaults
; EEPROM address is low byte of these addresses

NumberOfParameters	EQU	D'6'

ParameterValuesBase	EQU	0x0110

vPITCH_BEND		EQU	0x0110	; 0->120, pitch bend range in 20cent steps (up to 2 octaves)
vLFO_SPEED		EQU	0x0111	; 0 = slow, 127 = fast
vLFO_EXTRA_SPEED	EQU	0x0112	; 0 = slow, 127 = fast
vLFO_WAVE		EQU	0x0113	; 0 = sine, 1 = triangle, 2 = falling saw, 3 = rising saw, 4 = square
vLFO_DEPTH		EQU	0x0114	; 0 = 0V ptp, 127 = 8.8V ptp
vLEGATO_RETRIG		EQU	0x0115  ; 0 = off, 1 = on

vPARAMS_CHANGED		EQU	0x016F	; Flags for when parameters have changed
  bPITCH_BEND		EQU	0	; (to be checked every few seconds and written to EEPROM as necessary)
  bLFO_SPEED		EQU	1
  bLFO_EXTRA_SPEED	EQU	2
  bLFO_WAVE		EQU	3
  bLFO_DEPTH		EQU	4
  bLEGATO_RETRIG	EQU	5

; Max			EQU	0x016F

;==================================================
; Memory - bank 3


; Note + velocity stacks
NOTE_STACK		EQU	0x0190
;				0x0191
;				0x0192
;				0x0193
;				0x0194
;				0x0195
;				0x0196
;				0x0197
;				0x0198
;				0x0199
;				0x019A
;				0x019B
;				0x019C
;				0x019D
;				0x019E
NOTE_STACK_BOTTOM	EQU	0x019F
VELOCITY_STACK		EQU	0x01A0
;				0x01A1
;				0x01A2
;				0x01A3
;				0x01A4
;				0x01A5
;				0x01A6
;				0x01A7
;				0x01A8
;				0x01A9
;				0x01AA
;				0x01AB
;				0x01AC
;				0x01AD
;				0x01AE
VELOCITY_STACK_BOTTOM	EQU	0x01AF


; Max			EQU	0x01DF

; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0
; 0x00E0-0x00EF in bank 1
; 0x01E0-0x01EF in bank 3



;==================================================
; EEPROM

; EEPROM contains above bank 2 values at low byte of these addresses


EEPROM_FLAGS			EQU	0x00
  EEPROM_VALID			EQU	0x0	; Should be zero to indicate EEPROM contents is valid

EEPROM_ParameterValuesBase	EQU	0x10
