;*********************************************************************
; 3031 CONTROLLER ~ MIDI to CV Converter                             *
;*********************************************************************
; (c) 2005 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Maths subroutines (located in program bank 2)
;
;*********************************************************************

; ***********************************************************
; * Edit functions to use unrolled macros if space permits! *
; ***********************************************************

; Currently unrolled:
; FXM0707U
; FXM0808S
; FXM1507U
; FXM1608S
; FXM1608U
; FXD1608U

;---------------------------------------------------------
; 8x8 Bit Signed Fixed Point Multiply 8x8 > 16
;
; Input:  8 bit signed fixed point multiplicand in AARGB0
;         8 bit signed fixed point multiplier in BARGB0
; Use:    CALL FXM0808S
; Output: 16 bit signed fixed point product in AARGB0, AARGB1
; Result: AARG < AARG x BARG
;
; Max Timing: 12+69+2 = 83 clks B > 0
; 17+69+2 = 88 clks B < 0
; Min Timing: 12+47 = 59 clks
; 6 clks A = 0
; PM: 17+21+1 = 39 DM: 5
;---------------------------------------------------------
; 8x8 Bit Unsigned Fixed Point Multiply 8x8 > 16
;
; Input:  8 bit unsigned fixed point multiplicand in AARGB0
;         8 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM0808U
; Output: 16 bit unsigned fixed point product in AARGB0, AARGB1
; Result: AARG < AARG x BARG
;
; Max Timing: 1+70+2 = 73 clks
; Min Timing: 1+53 = 54 clks
; PM: 1+19+1 = 21 DM: 4
;---------------------------------------------------------
; 7x7 Bit Unsigned Fixed Point Divide 7x7 > 14
;
; Input:  7 bit unsigned fixed point multiplicand in AARGB0
;         7 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM0707U
; Output: 14 bit unsigned fixed point product in AARGB0, AARGB1
; Result: AARG < AARG x BARG
;
; Max Timing: 1+64+2 = 67 clks
; Min Timing: 1+47 = 48 clks
; PM: 1+21+1 = 23 DM: 4
;---------------------------------------------------------
;---------------------------------------------------------
; 16x8 Bit Signed Fixed Point Multiply 16x8 > 24
;
; Input:  16 bit signed fixed point multiplicand in AARGB0, AARGB1
;         8 bit signed fixed point multiplier in BARGB0
; Use:    CALL FXM1608S
; Output: 24 bit signed fixed point product in AARGB0, AARGB1, AARGB2
; Result: AARG < AARG x BARG
;
; Max Timing: 8+112+2 = 122 clks B > 0
; 14+112+2 = 128 clks B < 0
; Min Timing: 8+47 = 55 clks
; PM: 14+29+1 = 44 DM: 7
;---------------------------------------------------------
; 16x8 Bit Unsigned Fixed Point Multiply 16x8 > 24
;
; Input:  16 bit unsigned fixed point multiplicand in AARGB0, AARGB1
;         8 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM1608U
; Output: 24 bit unsigned fixed point product in AARGB0, AARGB1, AARGB2
; Result: AARG < AARG x BARG
;
; Max Timing: 5+119+2 = 126 clks
; Min Timing: 5+54 = 59 clks
; PM: 5+26+1 = 31 DM: 7
;---------------------------------------------------------
; 15x7 Bit Unsigned Fixed Point Multiply 15x7 > 22
;
; Input:  15 bit unsigned fixed point multiplicand in AARGB0, AARGB1
;         7 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM1507U
; Output: 22 bit unsigned fixed point product in AARGB0, AARGB1, AARGB2
; Result: AARG < AARG x BARG
;
; Max Timing: 5+107+2 = 114 clks
; Min Timing: 5+47 = 52 clks
; PM: 5+29+1 = 35 DM: 7
;---------------------------------------------------------
;---------------------------------------------------------
; 16/8 Bit Signed Fixed Point Divide 16/8 > 16.08 
;
; Input:  16 bit signed fixed point dividend in AARGB0, AARGB1
;         8 bit signed fixed point divisor in BARGB0
; Use:    CALL FXD1608S
; Output: 16 bit signed fixed point quotient in AARGB0, AARGB1
;         8 bit signed fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 23+163+5 = 191 clks A > 0, B > 0
;             24+163+13 = 200 clks A > 0, B < 0
;             27+163+13 = 203 clks A < 0, B > 0
;             28+163+5 = 196 clks A < 0, B < 0
;             7 clks A = 0
; Min Timing: 23+163+5 = 191 clks A > 0, B > 0
;             24+163+13 = 200 clks A > 0, B < 0
;             27+163+13 = 203 clks A < 0, B > 0
;             28+163+5 = 196 clks A < 0, B < 0
; PM: 28+42+12+34 = 116 DM: 8
;---------------------------------------------------------
; 16/8 Bit Unsigned Fixed Point Divide 16/8 > 16.08
;
; Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;         8 bit unsigned fixed point divisor in BARGB0
; Use:    CALL FXD1608U
; Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;         8 bit unsigned fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 1+291+2 = 294 clks
; Min Timing: 1+227+2 = 230 clks
; PM: 1+39+1 = 41 DM: 7
;---------------------------------------------------------
; 16/7 Bit Unsigned Fixed Point Divide 16/7 > 16.07
;
; Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;         7 bit unsigned fixed point divisor in BARGB0
; Use:    CALL FXD1607U
; Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;         7 bit unsigned fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 1+171+2 = 174 clks
; Min Timing: 1+171+2 = 174 clks
; PM: 1+39+1 = 41 DM: 5
;---------------------------------------------------------
; 15/7 Bit Unsigned Fixed Point Divide 15/7 > 15.07
;
; Input:  15 bit unsigned fixed point dividend in AARGB0, AARGB1
;         7 bit unsigned fixed point divisor in BARGB0
; Use:    CALL FXD1507U
; Output: 15 bit unsigned fixed point quotient in AARGB0, AARGB1
;         7 bit unsigned fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 1+163+2 = 166 clks
; Min Timing: 1+163+2 = 166 clks
; PM: 1+42+1 = 44 DM: 5
;---------------------------------------------------------
;---------------------------------------------------------
; 24/16 Bit Signed Fixed Point Divide 24/16 > 24.16
;
; Input:  24 bit fixed point dividend in AARGB0, AARGB1, AARGB2
;         16 bit fixed point divisor in BARGB0, BARGB1
; Use:    CALL FXD2416S
; Output: 24 bit fixed point quotient in AARGB0, AARGB1, AARGB2
;         16 bit fixed point remainder in REMB0, REMB1
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 25+403+5 = 433 clks A > 0, B > 0
; 29+403+19 = 451 clks A > 0, B < 0
; 32+403+19 = 454 clks A < 0, B > 0
; 36+403+5 = 444 clks A < 0, B < 0
; 9 clks A = 0
; Min Timing: 25+375+5 = 405 clks A > 0, B > 0
; 29+375+19 = 423 clks A > 0, B < 0
; 32+375+19 = 426 clks A < 0, B > 0
; 36+375+5 = 416 clks A < 0, B < 0
; PM: 36+117+18+48 = 219 DM: 10
;---------------------------------------------------------
; **** THIS ONE IS CURRENTLY COMMENTED OUT TO SAVE PM ****
; 24/16 Bit Unsigned Fixed Point Divide 24/16 > 24.16
;
; Input:  24 bit unsigned fixed point dividend in AARGB0, AARGB1, AARGB2
;         16 bit unsigned fixed point divisor in BARGB0, BARGB1
; Use:    CALL FXD2416U
; Output: 24 bit unsigned fixed point quotient in AARGB0, AARGB1, AARGB2
;         16 bit unsigned fixed point remainder in REMB0, REMB1
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 2+525+2 = 529 clks
; Min Timing: 2+497+2 = 501 clks
; PM: 2+169+1 = 172 DM: 8
;---------------------------------------------------------
; 23/15 Bit Unsigned Fixed Point Divide 23/15 > 23.15
;
; Input:  23 bit unsigned fixed point dividend in AARGB0, AARGB1, AARGB2
;         15 bit unsigned fixed point divisor in BARGB0, BARGB1
; Use:    CALL FXD2315U
; Output: 23 bit unsigned fixed point quotient in AARGB0, AARGB1, AARGB2
;         15 bit unsigned fixed point remainder in REMB0, REMB1
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 2+403+2 = 407 clks
; Min Timing: 2+375+2 = 379 clks
; PM: 2+117+1 = 120 DM: 7
;---------------------------------------------------------





; --------------------------------
; 08x08 Bit Multiplication Macros
; --------------------------------

SMUL0808L macro
; Max Timing: 3+10+5*9+8+3 = 69 clks
; Min Timing: 3+6*6+5+3 = 47 clks
; PM: 21 DM: 5
	MOVLW	0x07
	MOVWF	LOOPCOUNT
	MOVF	AARGB0,W
LOOPSM0808A
	RRF	BARGB0,F
	BTFSC	_C
	GOTO	LSM0808NA
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPSM0808A
	CLRF	AARGB0
	RETLW	0x00
LOOPSM0808
	RRF	BARGB0,F
	BTFSC	_C
	ADDWF	AARGB0,F
LSM0808NA
	RLF	SIGN,F
	RRF	AARGB0,F
	RRF	AARGB1,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPSM0808
	RLF	SIGN,F
	RRF	AARGB0,F
	RRF	AARGB1,F
endm

UMUL0808L macro
; Max Timing: 3+12+6*8+7 = 70 clks
; Min Timing: 3+7*6+5+3 = 53 clks
; PM: 19 DM: 4
	MOVLW	0x08
	MOVWF	LOOPCOUNT
	MOVF	AARGB0,W
LOOPUM0808A
	RRF	BARGB0,F
	BTFSC	_C
	GOTO	LUM0808NAP
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM0808A
	CLRF	AARGB0
	RETLW	0x00
LUM0808NAP
	BCF	_C
	GOTO	LUM0808NA
LOOPUM0808
	RRF	BARGB0,F
	BTFSC	_C
	ADDWF	AARGB0,F
LUM0808NA
	RRF	AARGB0,F
	RRF	AARGB1,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM0808
endm

UMUL0707L macro
; Max Timing: 3+12+5*8+7+2 = 64 clks
; Min Timing: 3+6*6+5+3 = 47 clks
; PM: 21 DM: 4
	MOVLW	0x07
	MOVWF	LOOPCOUNT
	MOVF	AARGB0,W
LOOPUM0707A
	RRF	BARGB0,F
	BTFSC	_C
	GOTO	LUM0707NAP
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM0707A
	CLRF	AARGB0
	RETLW	0x00
LUM0707NAP
	BCF	_C
	GOTO	LUM0707NA
LOOPUM0707
	RRF	BARGB0,F
	BTFSC	_C
	ADDWF	AARGB0,F
LUM0707NA
	RRF	AARGB0,F
	RRF	AARGB1,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM0707
	RRF	AARGB0,F
	RRF	AARGB1,F
endm

SMUL0808 macro
; Max Timing: 1+6+6*5+3 = 40 clks
; Min Timing: 1+14+3 = 18 clks
; PM: 1+2*7+5+6*5+3 = 53 DM: 5
	variable i = 0
	MOVF	AARGB0,W
	while i < 7
	BTFSC	BARGB0,i
	GOTO	SM0808NA#v(i)
	variable i = i + 1
	endw
	CLRF	AARGB0 ; if we get here, BARG = 0
	RETURN
SM0808NA0
	RLF	SIGN
	RRF	AARGB0
	RRF	AARGB1
	variable i = 1
	while i < 7
	BTFSC	BARGB0,i
	ADDWF	AARGB0
SM0808NA#v(i)
	RLF	SIGN
	RRF	AARGB0
	RRF	AARGB1
	variable i = i + 1
	endw
	RLF	SIGN
	RRF	AARGB0
	RRF	AARGB1
endm

UMUL0808 macro
; Max Timing: 2+5+7*4 = 35 clks
; Min Timing: 2+16+3 = 21 clks
; PM: 2+2*8+4+7*4 = 50 DM: 3
	variable i = 0
	BCF	_C ; clear carry for first right shift
	MOVF	AARGB0,W
	while i < 8
	BTFSC	BARGB0,i
	GOTO	UM0808NA#v(i)
	variable i = i + 1
	endw
	CLRF	AARGB0 ; if we get here, BARG = 0
	RETURN
UM0808NA0
	RRF	AARGB0,F
	RRF	AARGB1,F
	variable i = 1
	while i < 8
	BTFSC	BARGB0,i
	ADDWF	AARGB0,F
UM0808NA#v(i)
	RRF	AARGB0,F
	RRF	AARGB1,F
	variable i = i + 1
	endw
endm

UMUL0707 macro
; Max Timing: 2+5+6*4+2 = 33 clks
; Min Timing: 2+14+3 = 19 clks
; PM: 2+2*7+4+6*4+2 = 46 DM: 3
	variable i = 0
	BCF	_C ; clear carry for first right shift
	MOVF	AARGB0,W
	while i < 7
	BTFSC	BARGB0,i
	GOTO	UM0707NA#v(i)
	variable i = i + 1
	endw
	CLRF	AARGB0 ; if we get here, BARG = 0
	RETURN
UM0707NA0
	RRF	AARGB0,F
	RRF	AARGB1,F
	variable i = 1
	while i < 7
	BTFSC	BARGB0,i
	ADDWF	AARGB0,F
UM0707NA#v(i)
	RRF	AARGB0,F
	RRF	AARGB1,F
	variable i = i + 1
	endw
	RRF	AARGB0,F
	RRF	AARGB1,F
endm



;---------------------------------------------------------
; 8x8 Bit Signed Fixed Point Multiply 8x8 > 16
;
; Input:  8 bit signed fixed point multiplicand in AARGB0
;         8 bit signed fixed point multiplier in BARGB0
; Use:    CALL FXM0808S
; Output: 16 bit signed fixed point product in AARGB0, AARGB1
; Result: AARG < AARG x BARG
;
; Max Timing: 12+69+2 = 83 clks B > 0
; 17+69+2 = 88 clks B < 0
; Min Timing: 12+47 = 59 clks
; 6 clks A = 0
; PM: 17+21+1 = 39 DM: 5
;---------------------------------------------------------

FXM0808S
	CLRF	AARGB1 ; clear partial product
	CLRF	SIGN
	MOVF	AARGB0,W
	BTFSC	_Z
	RETLW	0x00
	XORWF	BARGB0,W
	MOVWF	TEMPB3
	BTFSC	TEMPB3,MSB
	COMF	SIGN,F
	BTFSS	BARGB0,MSB
	GOTO	M0808SOK
	COMF	BARGB0,F ; make multiplier BARG > 0
	INCF	BARGB0,F
	COMF	AARGB0,F
	INCF	AARGB0,F
	BTFSC	BARGB0,MSB
	GOTO	M0808SX
M0808SOK
	SMUL0808
	RETLW	0x00
M0808SX
	CLRF	AARGB1
	RLF	SIGN,W
	RRF	AARGB0,F
	RRF	AARGB1,F
	RETLW	0x00



;---------------------------------------------------------
; 8x8 Bit Unsigned Fixed Point Multiply 8x8 > 16
;
; Input:  8 bit unsigned fixed point multiplicand in AARGB0
;         8 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM0808U
; Output: 16 bit unsigned fixed point product in AARGB0, AARGB1
; Result: AARG < AARG x BARG
;
; Max Timing: 1+70+2 = 73 clks
; Min Timing: 1+53 = 54 clks
; PM: 1+19+1 = 21 DM: 4
;---------------------------------------------------------
FXM0808U
	CLRF	AARGB1 ; clear partial product
	UMUL0808L
	RETLW	0x00



;---------------------------------------------------------
; 7x7 Bit Unsigned Fixed Point Divide 7x7 > 14
;
; Input:  7 bit unsigned fixed point multiplicand in AARGB0
;         7 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM0707U
; Output: 14 bit unsigned fixed point product in AARGB0, AARGB1
; Result: AARG < AARG x BARG
;
; Max Timing: 1+64+2 = 67 clks
; Min Timing: 1+47 = 48 clks
; PM: 1+21+1 = 23 DM: 4
;---------------------------------------------------------
FXM0707U
	CLRF	AARGB1 ; clear partial product
	UMUL0707
	RETLW	0x00
	
	
	


;*********************************************************************


; --------------------------------
; 16x08 Bit Multiplication Macros
; --------------------------------

SMUL1608L macro
; Max Timing: 2+11+5*16+15+4 = 112 clks
; Min Timing: 2+6*6+5+4 = 47 clks
; PM: 29 DM: 7
	MOVLW	0x07
	MOVWF	LOOPCOUNT
LOOPSM1608A
	RRF	BARGB0,F
	BTFSC	_C
	GOTO	LSM1608NA
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPSM1608A
	CLRF	AARGB0
	CLRF	AARGB1
	RETLW	0x00
LOOPSM1608
	RRF	BARGB0,F
	BTFSS	_C
	GOTO	LSM1608NA
	MOVF	TEMPB1,W
	ADDWF	AARGB1,F
	MOVF	TEMPB0,W
	BTFSC	_C
	INCFSZ	TEMPB0,W
	ADDWF	AARGB0,F
LSM1608NA
	RLF	SIGN,W
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPSM1608
	RLF	SIGN,W
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
endm

UMUL1608L macro
; Max Timing: 2+13+6*15+14 = 119 clks
; Min Timing: 2+7*6+5+4 = 54 clks
; PM: 26 DM: 7
	MOVLW	0x08
	MOVWF	LOOPCOUNT
LOOPUM1608A
	RRF	BARGB0,F
	BTFSC	_C
	GOTO	LUM1608NAP
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM1608A
	CLRF	AARGB0
	CLRF	AARGB1
	RETLW	0x00
LUM1608NAP
	BCF	_C
	GOTO	LUM1608NA
LOOPUM1608
	RRF	BARGB0,F
	BTFSS	_C
	GOTO	LUM1608NA
	MOVF	TEMPB1,W
	ADDWF	AARGB1,F
	MOVF	TEMPB0,W
	BTFSC	_C
	INCFSZ	TEMPB0,W
	ADDWF	AARGB0,F
LUM1608NA
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM1608
endm

UMUL1507L macro
; Max Timing: 2+13+5*15+14+3 = 107 clks
; Min Timing: 2+6*6+5+4 = 47 clks
; PM: 29 DM: 7
	MOVLW	0x07
	MOVWF	LOOPCOUNT
LOOPUM1507A
	RRF	BARGB0,F
	BTFSC	_C
	GOTO	LUM1507NAP
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM1507A
	CLRF	AARGB0
	CLRF	AARGB1
	RETLW	0x00
LUM1507NAP
	BCF	_C
	GOTO	LUM1507NA
LOOPUM1507
	RRF	BARGB0,F
	BTFSS	_C
	GOTO	LUM1507NA
	MOVF	TEMPB1,W
	ADDWF	AARGB1,F
	MOVF	TEMPB0,W
	BTFSC	_C
	INCFSZ	TEMPB0,W
	ADDWF	AARGB0,F
LUM1507NA
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPUM1507
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
endm

SMUL1608 macro
; Max Timing: 3+6+6*11+3 = 78 clks
; Min Timing: 3+21+5 = 29 clks
; PM: 3+3*7+7+6*11+3 = 100 DM: 6
	variable i =0
	BTFSC	SIGN,MSB
	COMF	AARGB2,F
	RLF	SIGN,W
	while i < 7
	BTFSC	BARGB0,i
	GOTO	SM1608NA#v(i)
	BCF	AARGB2,7-i
	variable i = i + 1
	endw
	CLRF	AARGB0 ; if we get here, BARG = 0
	CLRF	AARGB1
	CLRF	AARGB2
	RETURN
SM1608NA0
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	variable i = 1
	while i < 7
	BTFSS	BARGB0,i
	GOTO	SM1608NA#v(i)
SM1608A#v(i)
	MOVF	TEMPB1,W
	ADDWF	AARGB1,F
	MOVF	TEMPB0,W
	BTFSC	_C
	INCFSZ	TEMPB0,W
	ADDWF	AARGB0,F
SM1608NA#v(i)
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	variable i = i + 1
	endw
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
endm

UMUL1608 macro
; Max Timing: 1+6+7*11 = 84 clks
; Min Timing: 1+2*8+4 = 21 clks
; PM: 1+2*8+4+6*7 = 63 DM: 4
	variable i = 0
	BCF	_C ; clear carry for first right shift
	while i < 8
	BTFSC	BARGB0,i
	GOTO	UM1608NA#v(i)
	variable i = i + 1
	endw
	CLRF	AARGB0 ; if we get here, BARG = 0
	CLRF	AARGB1
	RETURN
UM1608NA0
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	variable i = 1
	while i < 8
	BTFSS	BARGB0,i
	GOTO	UM1608NA#v(i)
UM1608A#v(i)
	MOVF	TEMPB1,W
	ADDWF	AARGB1,F
	MOVF	TEMPB0,W
	BTFSC	_C
	INCFSZ	TEMPB0,W
	ADDWF	AARGB0,F
UM1608NA#v(i)
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	variable i = i + 1
	endw
endm

UMUL1507 macro
; Max Timing: 7+6*12+4 = 83 clks
; Min Timing: 14+3 = 17 clks
; PM: 2*7+7+6*12+4 = 97 DM: 6
	variable i = 0
	BCF	_C ; clear carry for first right shift
	while i < 7
	BTFSC	BARGB0,i
	GOTO	UM1507NA#v(i)
	variable i = i + 1
	endw
	CLRF	AARGB0 ; if we get here, BARG = 0
	CLRF	AARGB1
	RETURN
UM1507NA0
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	variable i = 1
	while i < 7
	BTFSS	BARGB0,i
	GOTO	UM1507NA#v(i)
UM1507A#v(i)
	MOVF	TEMPB1,W
	ADDWF	AARGB1,F
	MOVF	TEMPB0,W
	BTFSC	_C
	INCFSZ	TEMPB0,W
	ADDWF	AARGB0,F
UM1507NA#v(i)
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	variable i = i + 1
	endw
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
endm



;---------------------------------------------------------
; 16x8 Bit Signed Fixed Point Multiply 16x8 > 24
;
; Input:  16 bit signed fixed point multiplicand in AARGB0, AARGB1
;         8 bit signed fixed point multiplier in BARGB0
; Use:    CALL FXM1608S
; Output: 24 bit signed fixed point product in AARGB0, AARGB1, AARGB2
; Result: AARG < AARG x BARG
;
; Max Timing: 8+112+2 = 122 clks B > 0
; 14+112+2 = 128 clks B < 0
; Min Timing: 8+47 = 55 clks
; PM: 14+29+1 = 44 DM: 7
;---------------------------------------------------------

FXM1608S
	CLRF	AARGB2 ; clear partial product
	CLRF	SIGN
	MOVF	AARGB0,W
	IORWF	AARGB1,W
	BTFSC	_Z
	RETLW	0x00
	MOVF	AARGB0,W
	XORWF	BARGB0,W
	MOVWF	TEMPB0
	BTFSC	TEMPB0,MSB
	COMF	SIGN,F
	BTFSS	BARGB0,MSB
	GOTO	M1608SOK
	COMF	BARGB0,F ; make multiplier BARG > 0
	INCF	BARGB0,F
	COMF	AARGB1,F
	COMF	AARGB0,F
	INCF	AARGB1,F
	BTFSC	_Z
	INCF	AARGB0,F
	BTFSC	BARGB0,MSB
	GOTO	M1608SX
M1608SOK
	MOVF	AARGB0,W
	MOVWF	TEMPB0
	MOVF	AARGB1,W
	MOVWF	TEMPB1
	SMUL1608L
	RETLW	0x00
M1608SX
	CLRF	AARGB2
	RLF	SIGN,W
	RRF	AARGB0,F
	RRF	AARGB1,F
	RRF	AARGB2,F
	RETLW	0x00
	

;---------------------------------------------------------
; 16x8 Bit Unsigned Fixed Point Multiply 16x8 > 24
;
; Input:  16 bit unsigned fixed point multiplicand in AARGB0, AARGB1
;         8 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM1608U
; Output: 24 bit unsigned fixed point product in AARGB0, AARGB1, AARGB2
; Result: AARG < AARG x BARG
;
; Max Timing: 5+119+2 = 126 clks
; Min Timing: 5+54 = 59 clks
; PM: 5+26+1 = 31 DM: 7
;---------------------------------------------------------

FXM1608U
	CLRF	AARGB2 ; clear partial product
	MOVF	AARGB0,W
	MOVWF	TEMPB0
	MOVF	AARGB1,W
	MOVWF	TEMPB1
	UMUL1608
	RETLW	0x00
	
	
;---------------------------------------------------------
; 15x7 Bit Unsigned Fixed Point Multiply 15x7 > 22
;
; Input:  15 bit unsigned fixed point multiplicand in AARGB0, AARGB1
;         7 bit unsigned fixed point multiplier in BARGB0
; Use:    CALL FXM1507U
; Output: 22 bit unsigned fixed point product in AARGB0, AARGB1, AARGB2
; Result: AARG < AARG x BARG
;
; Max Timing: 5+107+2 = 114 clks
; Min Timing: 5+47 = 52 clks
; PM: 5+29+1 = 35 DM: 7
;---------------------------------------------------------

FXM1507U
	CLRF	AARGB2 ; clear partial product
	MOVF	AARGB0,W
	MOVWF	TEMPB0
	MOVF	AARGB1,W
	MOVWF	TEMPB1
	UMUL1507
	RETLW	0x00
	





;*********************************************************************


; --------------------------------
; 16/8 Bit Division Macros
; --------------------------------

SDIV1608L macro 
; Max Timing: 3+5+2+5*11+10+10+6*11+10+2 = 163 clks 
; Min Timing: 3+5+2+5*11+10+10+6*11+10+2 = 163 clks 
; PM: 42 DM: 5 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	RLF	AARGB0, F 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	MOVLW	6 
	MOVWF	LOOPCOUNT 
LOOPS1608A
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPS1608A 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	MOVLW	7 
	MOVWF	LOOPCOUNT 
LOOPS1608B
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB1,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPS1608B 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
endm 
UDIV1608L macro 
; Max Timing: 2+7*12+11+3+7*24+23 = 291 clks 
; Min Timing: 2+7*11+10+3+7*17+16 = 227 clks 
; PM: 39 DM: 7 
	MOVLW	8 
	MOVWF	LOOPCOUNT 
LOOPU1608A
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	BTFSC	_C 
	GOTO	UOK68A 
	ADDWF	REMB0, F 
	BCF	_C 
UOK68A
	RLF	AARGB0, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPU1608A 
	CLRF	TEMP 
	MOVLW	8 
	MOVWF	LOOPCOUNT
LOOPU1608B
	RLF	AARGB1,W 
	RLF	REMB0, F 
	RLF	TEMP, F 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	CLRF	AARGB5 
	CLRW 
	BTFSS	_C 
	INCFSZ	AARGB5,W 
	SUBWF	TEMP, F 
	BTFSC	_C 
	GOTO	UOK68B 
	MOVF	BARGB0,W 
	ADDWF	REMB0, F 
	CLRF	AARGB5 
	CLRW 
	BTFSC	_C 
	INCFSZ	AARGB5,W 
	ADDWF	TEMP, F 
	BCF	_C 
UOK68B
	RLF	AARGB1, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPU1608B 
endm 
UDIV1607L macro 
; Max Timing: 7+6*11+10+10+6*11+10+2 = 171 clks 
; Min Timing: 7+6*11+10+10+6*11+10+2 = 171 clks 
; PM: 39 DM: 5 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	RLF	AARGB0, F 
	MOVLW	7 
	MOVWF	LOOPCOUNT 
LOOPU1607A
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPU1607A 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	MOVLW	7 
	MOVWF	LOOPCOUNT 
LOOPU1607B
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB1,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPU1607B 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
endm 
UDIV1507L macro 
; Max Timing: 3+5+2+5*11+10+10+6*11+10+2 = 163 clks 
; Min Timing: 3+5+2+5*11+10+10+6*11+10+2 = 163 clks 
; PM: 42 DM: 5 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	RLF	AARGB0, F 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	MOVLW	6 
	MOVWF	LOOPCOUNT 
LOOPU1507A
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPU1507A 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	MOVLW	7 
	MOVWF	LOOPCOUNT 
LOOPU1507B
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB1,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	DECFSZ	LOOPCOUNT, F 
	GOTO	LOOPU1507B 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
endm 
SDIV1608 macro 
; Max Timing: 3+5+14*8+2 = 122 clks 
; Min Timing: 3+5+14*8+2 = 122 clks 
; PM: 122 DM: 4 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	RLF	AARGB0, F 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	variable i = 2 
	while i < 8 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	variable i = i + 1 
	endw 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	variable i = 9 
	while i < D'16' 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB1,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	variable i = i + 1 
	endw 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
endm 
UDIV1608 macro 
; restore = 9/21 clks, nonrestore = 8/14 clks 
; Max Timing: 8*9+1+8*21 = 241 clks 
; Min Timing: 8*8+1+8*14 = 177 clks 
; PM: 241 DM: 6 
	variable i = 0 
	while i < 8 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	BTFSC	_C 
	GOTO	UOK68#v(i) 
	ADDWF	REMB0, F 
	BCF	_C 
UOK68#v(i)
	RLF	AARGB0, F 
	variable i = i + 1 
	endw 
	CLRF	TEMP 
	variable i = 8 
	while i < D'16' 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	RLF	TEMP, F 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	CLRF	AARGB5 
	CLRW 
	BTFSS	_C 
	INCFSZ	AARGB5,W 
	SUBWF	TEMP, F 
	BTFSC	_C 
	GOTO	UOK68#v(i) 
	MOVF	BARGB0,W 
	ADDWF	REMB0, F 
	CLRF	AARGB5 
	CLRW 
	BTFSC	_C 
	INCFSZ	AARGB5,W 
	ADDWF	TEMP, F 
	BCF	_C 
UOK68#v(i)
	RLF	AARGB1, F 
	variable i = i + 1 
	endw 
endm 
UDIV1607 macro 
; Max Timing: 5+15*8+2 = 127 clks 
; Min Timing: 5+15*8+2 = 127 clks 
; PM: 127 DM: 4 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	RLF	AARGB0, F 
	variable i = 1 
	while i < 8 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	variable i = i + 1 
	endw 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	variable i = 9 
	while i < D'16' 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB1,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	variable i = i + 1 
	endw 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
endm 
UDIV1507 macro 
; Max Timing: 3+5+14*8+2 = 122 clks 
; Min Timing: 3+5+14*8+2 = 122 clks 
; PM: 122 DM: 4 
	MOVF	BARGB0,W 
	SUBWF	REMB0, F 
	RLF	AARGB0, F 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	variable i = 2 
	while i < 8 
	RLF	AARGB0,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB0, F 
	variable i = i + 1 
	endw 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB0,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB0,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	variable i = 9 
	while i < D'16' 
	RLF	AARGB1,W 
	RLF	REMB0, F 
	MOVF	BARGB0,W 
	BTFSC	AARGB1,LSB 
	SUBWF	REMB0, F 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
	RLF	AARGB1, F 
	variable i = i + 1 
	endw 
	BTFSS	AARGB1,LSB 
	ADDWF	REMB0, F 
endm 



;---------------------------------------------------------
; 16/8 Bit Signed Fixed Point Divide 16/8 > 16.08 
;
; Input:  16 bit signed fixed point dividend in AARGB0, AARGB1
;         8 bit signed fixed point divisor in BARGB0
; Use:    CALL FXD1608S
; Output: 16 bit signed fixed point quotient in AARGB0, AARGB1
;         8 bit signed fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 23+163+5 = 191 clks A > 0, B > 0
;             24+163+13 = 200 clks A > 0, B < 0
;             27+163+13 = 203 clks A < 0, B > 0
;             28+163+5 = 196 clks A < 0, B < 0
;             7 clks A = 0
; Min Timing: 23+163+5 = 191 clks A > 0, B > 0
;             24+163+13 = 200 clks A > 0, B < 0
;             27+163+13 = 203 clks A < 0, B > 0
;             28+163+5 = 196 clks A < 0, B < 0
; PM: 28+42+12+34 = 116 DM: 8
;---------------------------------------------------------
FXD1608S
	CLRF	SIGN 
	CLRF	REMB0		; clear partial remainder 
	MOVF	AARGB0,W 
	IORWF	AARGB1,W 
	BTFSC	_Z 
	RETLW	0x00 
	MOVF	AARGB0,W 
	XORWF	BARGB0,W 
	MOVWF	TEMP
	BTFSC	TEMP,MSB 
	COMF	SIGN,F 
	CLRF	TEMPB3		; clear exception flag 
	BTFSS	BARGB0,MSB	; if MSB set, negate BARG 
	GOTO	CA1608S 
	COMF	BARGB0, F 
	INCF	BARGB0, F 
CA1608S
	BTFSS	AARGB0,MSB	; if MSB set, negate AARG 
	GOTO	C1608SX 
	COMF	AARGB1, F 
	COMF	AARGB0, F 
	INCF	AARGB1, F 
	BTFSC	_Z 
	INCF	AARGB0, F 
C1608SX
	MOVF	AARGB0,W 
	IORWF	BARGB0,W 
	MOVWF	TEMP 
	BTFSC	TEMP,MSB 
	GOTO	C1608SX1 
C1608S
	SDIV1608 
	BTFSC	TEMPB3,LSB	; test exception flag 
	GOTO	C1608SX4 
C1608SOK
	BTFSS	SIGN,MSB 
	RETLW	0x00 
	COMF	AARGB1, F 
	COMF	AARGB0, F 
	INCF	AARGB1, F 
	BTFSC	_Z 
	INCF	AARGB0, F 
	COMF	REMB0, F 
	INCF	REMB0, F 
	RETLW	0x00 
C1608SX1
	BTFSS	BARGB0,MSB	; test BARG exception 
	GOTO	C1608SX3 
	BTFSC	AARGB0,MSB	; test AARG exception 
	GOTO	C1608SX2 
	MOVF	AARGB1,W 
	MOVWF	REMB0 
	BCF	REMB0,MSB 
	RLF	AARGB1,F 
	RLF	AARGB0,F 
	MOVF	AARGB0,W 
	MOVWF	AARGB1 
	CLRF	AARGB0 
	GOTO	C1608SOK 
C1608SX2
	CLRF	AARGB1		; quotient = 1, remainder = 0 
	INCF	AARGB1,F 
	CLRF	AARGB0 
	RETLW	0x00 
C1608SX3
	COMF	AARGB0,F	; numerator = 0x7FFF + 1 
	COMF	AARGB1,F 
	INCF	TEMPB3,F 
	GOTO	C1608S 
C1608SX4
	INCF	REMB0,F		; increment remainder and test for 
	MOVF	BARGB0,W	; overflow 
	SUBWF	REMB0,W 
	BTFSS	_Z 
	GOTO	C1608SOK 
	CLRF	REMB0		; if remainder overflow, clear 
	INCF	AARGB1,F	; remainder, increment quotient and 
	BTFSC	_Z 
	INCF	AARGB0,F	; test for overflow exception 
	BTFSS	AARGB0,MSB 
	GOTO	C1608SOK 
	BSF	FPFLAGS,NAN 
	RETLW	0xFF 



;---------------------------------------------------------
; 16/8 Bit Unsigned Fixed Point Divide 16/8 > 16.08
;
; Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;         8 bit unsigned fixed point divisor in BARGB0
; Use:    CALL FXD1608U
; Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;         8 bit unsigned fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 1+291+2 = 294 clks
; Min Timing: 1+227+2 = 230 clks
; PM: 1+39+1 = 41 DM: 7
;---------------------------------------------------------
FXD1608U
	CLRF	REMB0 
	UDIV1608
	RETLW	0x00 



;---------------------------------------------------------
; 16/7 Bit Unsigned Fixed Point Divide 16/7 > 16.07
;
; Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;         7 bit unsigned fixed point divisor in BARGB0
; Use:    CALL FXD1607U
; Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;         7 bit unsigned fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 1+171+2 = 174 clks
; Min Timing: 1+171+2 = 174 clks
; PM: 1+39+1 = 41 DM: 5
;---------------------------------------------------------
FXD1607U
	CLRF	REMB0 
	UDIV1607L 
	RETLW	0x00 



;---------------------------------------------------------
; 15/7 Bit Unsigned Fixed Point Divide 15/7 > 15.07
;
; Input:  15 bit unsigned fixed point dividend in AARGB0, AARGB1
;         7 bit unsigned fixed point divisor in BARGB0
; Use:    CALL FXD1507U
; Output: 15 bit unsigned fixed point quotient in AARGB0, AARGB1
;         7 bit unsigned fixed point remainder in REMB0
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 1+163+2 = 166 clks
; Min Timing: 1+163+2 = 166 clks
; PM: 1+42+1 = 44 DM: 5
;---------------------------------------------------------
FXD1507U
	CLRF	REMB0 
	UDIV1507L
	RETLW	0x00 




;*********************************************************************


; --------------------------------
; 24/16 Bit Division Macros
; --------------------------------

SDIV2416L macro
; Max Timing: 9+6*17+16+16+6*17+16+16+6*17+16+8 = 403 clks
; Min Timing: 9+6*16+15+15+6*16+15+15+6*16+15+3 = 375 clks
; PM: 7+2*40+22+8 = 117 DM: 7
	MOVF	BARGB1,W
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	RLF	AARGB0,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPS2416A
	RLF	AARGB0,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB0,LSB
	GOTO	SADD46LA
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	SOK46LA
SADD46LA
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
SOK46LA	
	RLF	AARGB0,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPS2416A
	RLF	AARGB1,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB0,LSB
	GOTO	SADD46L8
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	SOK46L8
SADD46L8
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
SOK46L8
	RLF	AARGB1,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPS2416B
	RLF	AARGB1,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB1,LSB
	GOTO	SADD46LB
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	SOK46LB
SADD46LB
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
SOK46LB
	RLF	AARGB1,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPS2416B
	RLF	AARGB2,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB1,LSB
	GOTO	SADD46L16
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	SOK46L16
SADD46L16
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
SOK46L16
	RLF	AARGB2,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPS2416C
	RLF	AARGB2,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB2,LSB
	GOTO	SADD46LC
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	SOK46LC
SADD46LC
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
SOK46LC
	RLF	AARGB2,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPS2416C
	BTFSC	AARGB2,LSB
	GOTO	SOK46L
	MOVF	BARGB1,W
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
SOK46L
endm
UDIV2416L macro
; Max Timing: 16+6*22+21+21+6*22+21+21+6*22+21+8 = 525 clks
; Min Timing: 16+6*21+20+20+6*21+20+20+6*21+20+3 = 497 clks
; PM: 14+31+27+31+27+31+8 = 169 DM: 8
	CLRF	TEMP
	RLF	AARGB0,W
	RLF	REMB1,F
	MOVF	BARGB1,W
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	CLRW
	BTFSS	_C
	MOVLW	1
	SUBWF	TEMP,F
	RLF	AARGB0,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPU2416A
	RLF	AARGB0,W
	RLF	REMB1,F
	RLF	REMB0,F
	RLF	TEMP,F
	MOVF	BARGB1,W
	BTFSS	AARGB0,LSB
	GOTO	UADD46LA
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	CLRW
	BTFSS	_C
	MOVLW	1
	SUBWF	TEMP,F
	GOTO	UOK46LA
UADD46LA
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
	CLRW
	BTFSC	_C
	MOVLW	1
	ADDWF	TEMP,F
UOK46LA
	RLF	AARGB0,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPU2416A
	RLF	AARGB1,W
	RLF	REMB1,F
	RLF	REMB0,F
	RLF	TEMP,F
	MOVF	BARGB1,W
	BTFSS	AARGB0,LSB
	GOTO	UADD46L8
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	CLRW
	BTFSS	_C
	MOVLW	1
	SUBWF	TEMP,F
	GOTO	UOK46L8
UADD46L8
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
	CLRW
	BTFSC	_C
	MOVLW	1
	ADDWF	TEMP,F
UOK46L8
	RLF	AARGB1,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPU2416B
	RLF	AARGB1,W
	RLF	REMB1,F
	RLF	REMB0,F
	RLF	TEMP,F
	MOVF	BARGB1,W
	BTFSS	AARGB1,LSB
	GOTO	UADD46LB
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	CLRW
	BTFSS	_C
	MOVLW	1
	SUBWF	TEMP,F
	GOTO	UOK46LB
UADD46LB
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
	CLRW
	BTFSC	_C
	MOVLW	1
	ADDWF	TEMP,F
UOK46LB
	RLF	AARGB1,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPU2416B
	RLF	AARGB2,W
	RLF	REMB1,F
	RLF	REMB0,F
	RLF	TEMP,F
	MOVF	BARGB1,W
	BTFSS	AARGB1,LSB
	GOTO	UADD46L16
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	CLRW
	BTFSS	_C
	MOVLW	1
	SUBWF	TEMP,F
	GOTO	UOK46L16
UADD46L16
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
	CLRW
	BTFSC	_C
	MOVLW	1
	ADDWF	TEMP,F
UOK46L16
	RLF	AARGB2,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPU2416C
	RLF	AARGB2,W
	RLF	REMB1,F
	RLF	REMB0,F
	RLF	TEMP,F
	MOVF	BARGB1,W
	BTFSS	AARGB2,LSB
	GOTO	UADD46LC
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	CLRW
	BTFSS	_C
	MOVLW	1
	SUBWF	TEMP,F
	GOTO	UOK46LC
UADD46LC
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
	CLRW
	BTFSC	_C
	MOVLW	1
	ADDWF	TEMP,F
UOK46LC
	RLF	AARGB2,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPU2416C
	BTFSC	AARGB2,LSB
	GOTO	UOK46L
	MOVF	BARGB1,W
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
UOK46L
endm
UDIV2315L macro
; Max Timing: 9+6*17+16+16+6*17+16+16+6*17+16+8 = 403 clks
; Min Timing: 9+6*16+15+15+6*16+15+15+6*16+15+3 = 375 clks
; PM: 7+2*40+22+8 = 117 DM: 7
	MOVF	BARGB1,W
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	RLF	AARGB0,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPU2315A
	RLF	AARGB0,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB0,LSB
	GOTO	UADD35LA
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	UOK35LA
UADD35LA
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
UOK35LA
	RLF	AARGB0,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPU2315A
	RLF	AARGB1,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB0,LSB
	GOTO	UADD35L8
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	UOK35L8
UADD35L8
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
UOK35L8
	RLF	AARGB1,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPU2315B
	RLF	AARGB1,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB1,LSB
	GOTO	UADD35LB
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	UOK35LB
UADD35LB
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
UOK35LB
	RLF	AARGB1,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPU2315B
	RLF	AARGB2,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB1,LSB
	GOTO	UADD35L16
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	UOK35L16
UADD35L16
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
UOK35L16
	RLF	AARGB2,F
	MOVLW	7
	MOVWF	LOOPCOUNT
LOOPU2315C
	RLF	AARGB2,W
	RLF	REMB1,F
	RLF	REMB0,F
	MOVF	BARGB1,W
	BTFSS	AARGB2,LSB
	GOTO	UADD35LC
	SUBWF	REMB1,F
	MOVF	BARGB0,W
	BTFSS	_C
	INCFSZ	BARGB0,W
	SUBWF	REMB0,F
	GOTO	UOK35LC
UADD35LC
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
UOK35LC
	RLF	AARGB2,F
	DECFSZ	LOOPCOUNT,F
	GOTO	LOOPU2315C
	BTFSC	AARGB2,LSB
	GOTO	UOK35L
	MOVF	BARGB1,W
	ADDWF	REMB1,F
	MOVF	BARGB0,W
	BTFSC	_C
	INCFSZ	BARGB0,W
	ADDWF	REMB0,F
UOK35L
endm



;---------------------------------------------------------
; 24/16 Bit Signed Fixed Point Divide 24/16 > 24.16
;
; Input:  24 bit fixed point dividend in AARGB0, AARGB1, AARGB2
;         16 bit fixed point divisor in BARGB0, BARGB1
; Use:    CALL FXD2416S
; Output: 24 bit fixed point quotient in AARGB0, AARGB1, AARGB2
;         16 bit fixed point remainder in REMB0, REMB1
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 25+403+5 = 433 clks A > 0, B > 0
; 29+403+19 = 451 clks A > 0, B < 0
; 32+403+19 = 454 clks A < 0, B > 0
; 36+403+5 = 444 clks A < 0, B < 0
; 9 clks A = 0
; Min Timing: 25+375+5 = 405 clks A > 0, B > 0
; 29+375+19 = 423 clks A > 0, B < 0
; 32+375+19 = 426 clks A < 0, B > 0
; 36+375+5 = 416 clks A < 0, B < 0
; PM: 36+117+18+48 = 219 DM: 10
;---------------------------------------------------------
FXD2416S
	CLRF	SIGN
	CLRF	REMB0		; clear partial remainder
	CLRF	REMB1
	MOVF	AARGB0,W
	IORWF	AARGB1,W
	IORWF	AARGB2,W
	BTFSC	_Z
	RETLW	0x00
	MOVF	AARGB0,W
	XORWF	BARGB0,W
	MOVWF	TEMP
	BTFSC	TEMP,MSB
	COMF	SIGN,F
	CLRF	TEMPB3		; clear exception flag
	BTFSS	BARGB0,MSB	; if MSB set, negate BARG
	GOTO	CA2416S
	COMF	BARGB1,F
	COMF	BARGB0,F
	INCF	BARGB1,F
	BTFSC	_Z
	INCF	BARGB0,F
CA2416S
	BTFSS	AARGB0,MSB	; if MSB set, negate AARG
	GOTO	C2416SX
	COMF	AARGB2,F
	COMF	AARGB1,F
	COMF	AARGB0,F
	INCF	AARGB2,F
	BTFSC	_Z
	INCF	AARGB1,F
	BTFSC	_Z
	INCF	AARGB0,F
C2416SX
	MOVF	AARGB0,W
	IORWF	BARGB0,W
	MOVWF	TEMP
	BTFSC	TEMP,MSB
	GOTO	C2416SX1
C2416S
	SDIV2416L
	BTFSC	TEMPB3,LSB	; test exception flag
	GOTO	C2416SX4
C2416SOK
	BTFSS	SIGN,MSB
	RETLW	0x00
	COMF	AARGB2,F
	COMF	AARGB1,F
	COMF	AARGB0,F
	INCF	AARGB2,F
	BTFSC	_Z
	INCF	AARGB1,F
	BTFSC	_Z
	INCF	AARGB0,F
	COMF	REMB1,F
	COMF	REMB0,F
	INCF	REMB1,F
	BTFSC	_Z
	INCF	REMB0,F
	RETLW	0x00
C2416SX1
	BTFSS	BARGB0,MSB	; test BARG exception
	GOTO	C2416SX3
	BTFSC	AARGB0,MSB	; test AARG exception
	GOTO	C2416SX2
	MOVF	AARGB1,W
	MOVWF	REMB0
	MOVF	AARGB2,W
	MOVWF	REMB1
	BCF	REMB0,MSB
	RLF	AARGB1,F
	RLF	AARGB0,F
	MOVF	AARGB0,W
	MOVWF	AARGB2
	CLRF	AARGB0
	CLRF	AARGB1
	GOTO	C2416SOK
C2416SX2
	CLRF	AARGB2	; quotient = 1, remainder = 0
	INCF	AARGB2,F
	CLRF	AARGB1
	CLRF	AARGB0
	RETLW	0x00
C2416SX3
	COMF	AARGB0,F	; numerator = 0x7FFFFF + 1
	COMF	AARGB1,F
	COMF	AARGB2,F
	INCF	TEMPB3,F
	GOTO	C2416S
C2416SX4
	INCF	REMB1,F	; increment remainder and test for
	BTFSC	_Z
	INCF	REMB0,F
	MOVF	BARGB1,W	; overflow
	SUBWF	REMB1,W
	BTFSS	_Z
	GOTO	C2416SOK
	MOVF	BARGB0,W	; overflow
	SUBWF	REMB0,W
	BTFSS	_Z
	GOTO	C2416SOK
	CLRF	REMB0	; if remainder overflow, clear
	CLRF	REMB1
	INCF	AARGB2,F	; remainder, increment quotient and
	BTFSC	_Z
	INCF	AARGB1,F	; test for overflow exception
	BTFSC	_Z
	INCF	AARGB0,F
	BTFSS	AARGB0,MSB
	GOTO	C2416SOK
	BSF	FPFLAGS,NAN
	RETLW	0xFF



;---------------------------------------------------------
; 24/16 Bit Unsigned Fixed Point Divide 24/16 > 24.16
;
; Input:  24 bit unsigned fixed point dividend in AARGB0, AARGB1, AARGB2
;         16 bit unsigned fixed point divisor in BARGB0, BARGB1
; Use:    CALL FXD2416U
; Output: 24 bit unsigned fixed point quotient in AARGB0, AARGB1, AARGB2
;         16 bit unsigned fixed point remainder in REMB0, REMB1
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 2+525+2 = 529 clks
; Min Timing: 2+497+2 = 501 clks
; PM: 2+169+1 = 172 DM: 8
;---------------------------------------------------------
;FXD2416U
;	CLRF	REMB0
;	CLRF	REMB1
;	UDIV2416L
;	RETLW	0x00



;---------------------------------------------------------
; 23/15 Bit Unsigned Fixed Point Divide 23/15 > 23.15
;
; Input:  23 bit unsigned fixed point dividend in AARGB0, AARGB1, AARGB2
;         15 bit unsigned fixed point divisor in BARGB0, BARGB1
; Use:    CALL FXD2315U
; Output: 23 bit unsigned fixed point quotient in AARGB0, AARGB1, AARGB2
;         15 bit unsigned fixed point remainder in REMB0, REMB1
; Result: AARG, REM < AARG / BARG
;
; Max Timing: 2+403+2 = 407 clks
; Min Timing: 2+375+2 = 379 clks
; PM: 2+117+1 = 120 DM: 7
;---------------------------------------------------------
FXD2315U
	CLRF	REMB0
	CLRF	REMB1
	UDIV2315L
	RETLW	0x00
	
	
	
	
	
;**********************************************************************************************
